@@ -3,7 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2005 James Aylett
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
  * Copyright 2009 Frank J Bruzzaniti
  *
  * This program is free software; you can redistribute it and/or
@@ -27,18 +27,19 @@
 #include <algorithm>
 #include <fstream>
 #include <iostream>
+#include <limits>
 #include <string>
 #include <map>
 #include <vector>
 
 #include <sys/types.h>
 #include "safeunistd.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include "safefcntl.h"
 #include "safeerrno.h"
-#include <time.h>
+#include <ctime>
 
 #include <xapian.h>
 
@@ -51,6 +52,7 @@
 #include "myhtmlparse.h"
 #include "runfilter.h"
 #include "sample.h"
+#include "str.h"
 #include "stringutils.h"
 #include "utf8convert.h"
 #include "utils.h"
@@ -74,6 +76,8 @@ using namespace std;
 
 static bool skip_duplicates = false;
 static bool follow_symlinks = false;
+static bool spelling = false;
+
 static string dbpath;
 static string root;
 static string indexroot;
@@ -84,6 +88,8 @@ static Xapian::TermGenerator indexer;
 static vector<bool> updated;
 static string tmpdir;
 
+static time_t last_mod_max;
+
 inline static bool
 p_notalnum(unsigned int c)
 {
@@ -232,9 +238,33 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
     if (urlterm.length() > MAX_SAFE_TERM_LENGTH)
 	urlterm = hash_long_term(urlterm, MAX_SAFE_TERM_LENGTH);
 
-    if (skip_duplicates && db.term_exists(urlterm)) {
-	cout << "duplicate. Ignored." << endl;
-	return;
+    Xapian::docid did = 0; 
+    if (skip_duplicates) {
+        if (db.term_exists(urlterm)) {
+	    cout << "duplicate. Ignored." << endl;
+	    return;
+	}
+    } else {
+	// If last_mod > last_mod_max, we know for sure that the file is new
+	// or updated.
+	if (last_mod <= last_mod_max) {
+	    Xapian::PostingIterator p = db.postlist_begin(urlterm);
+	    if (p != db.postlist_end(urlterm)) {
+		did = *p;
+		Xapian::Document doc = db.get_document(did);
+		string value = doc.get_value(VALUE_LASTMOD);
+		time_t old_last_mod = binary_string_to_int(value);
+		if (last_mod <= old_last_mod) {
+		    cout << "Already indexed." << endl;
+		    // The docid should be in updated - the only valid
+		    // exception is if the URL was long and hashed to the
+		    // same URL as an existing document indexed in the same
+		    // batch.
+		    if (usual(did < updated.size())) updated[did] = true;
+		    return;
+		}
+	    }
+	}
     }
 
     string md5;
@@ -396,7 +426,10 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	} else if (startswith(tail, "spreadsheetml.")) {
 	    args = " xl/sharedStrings.xml";
 	} else if (startswith(tail, "presentationml.")) {
-	    args = " ppt/slides/slide*.xml";
+	    // unzip returns exit code 11 if a file to extract wasn't found
+	    // which we want to ignore, because there may be no notesSlides
+	    // or comments.
+	    args = " ppt/slides/slide*.xml ppt/notesSlides/notesSlide*.xml ppt/comments/comment*.xml 2>/dev/null||test $? = 11";
 	} else {
 	    // Don't know how to index this type.
 	    cout << "unknown Office 2007 MIME subtype - skipping" << endl;
@@ -562,10 +595,14 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	record += "\ncaption=" + generate_sample(title, TITLE_SIZE);
     }
     record += "\ntype=" + mimetype;
-    if (last_mod != (time_t)-1)
-	record += "\nmodtime=" + long_to_string(last_mod);
-    if (size)
-	record += "\nsize=" + long_to_string(size);
+    if (last_mod != (time_t)-1) {
+	record += "\nmodtime=";
+	record += str(last_mod);
+    }
+    if (size) {
+	record += "\nsize=";
+	record += str(size);
+    }
     newdocument.set_data(record);
 
     // Index the title, document text, and keywords.
@@ -615,7 +652,8 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
     newdocument.add_term(urlterm); // Url
 
     // Add last_mod as a value to allow "sort by date".
-    newdocument.add_value(VALUE_LASTMOD, int_to_binary_string((uint32_t)last_mod));
+    newdocument.add_value(VALUE_LASTMOD,
+			  int_to_binary_string((uint32_t)last_mod));
 
     // Add MD5 as a value to allow duplicate documents to be collapsed together.
     newdocument.add_value(VALUE_MD5, md5);
@@ -623,7 +661,15 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
     if (!skip_duplicates) {
 	// If this document has already been indexed, update the existing
 	// entry.
-	Xapian::docid did = db.replace_document(urlterm, newdocument);
+	if (did) {
+	    // We already found out the document id above.
+	    db.replace_document(did, newdocument);
+	} else if (last_mod <= last_mod_max) {
+	    // We checked for the UID term and didn't find it.
+	    did = db.add_document(newdocument);
+	} else {
+	    did = db.replace_document(urlterm, newdocument);
+	}
 	if (did < updated.size()) {
 	    updated[did] = true;
 	    cout << "updated." << endl;
@@ -759,6 +805,7 @@ main(int argc, char **argv)
 	{ "depth-limit",required_argument,	NULL, 'l' },
 	{ "follow",	no_argument,		NULL, 'f' },
 	{ "stemmer",	required_argument,	NULL, 's' },
+	{ "spelling",	no_argument,		NULL, 'S' },
 	{ 0, 0, NULL, 0 }
     };
 
@@ -768,17 +815,21 @@ main(int argc, char **argv)
     // Plain text:
     mime_map["txt"] = "text/plain";
     mime_map["text"] = "text/plain";
+
     // HTML:
     mime_map["html"] = "text/html";
     mime_map["htm"] = "text/html";
     mime_map["shtml"] = "text/html";
     mime_map["php"] = "text/html"; // Our HTML parser knows to ignore PHP code.
+
     // PDF:
     mime_map["pdf"] = "application/pdf";
+
     // PostScript:
     mime_map["ps"] = "application/postscript";
     mime_map["eps"] = "application/postscript";
     mime_map["ai"] = "application/postscript";
+
     // OpenDocument:
     // FIXME: need to find sample documents to test all of these.
     mime_map["odt"] = "application/vnd.oasis.opendocument.text";
@@ -798,6 +849,7 @@ main(int argc, char **argv)
     mime_map["otf"] = "application/vnd.oasis.opendocument.formula-template";
     mime_map["oti"] = "application/vnd.oasis.opendocument.image-template";
     mime_map["oth"] = "application/vnd.oasis.opendocument.text-web";
+
     // OpenOffice/StarOffice documents:
     mime_map["sxc"] = "application/vnd.sun.xml.calc";
     mime_map["stc"] = "application/vnd.sun.xml.calc.template";
@@ -809,6 +861,7 @@ main(int argc, char **argv)
     mime_map["sxw"] = "application/vnd.sun.xml.writer";
     mime_map["sxg"] = "application/vnd.sun.xml.writer.global";
     mime_map["stw"] = "application/vnd.sun.xml.writer.template";
+
     // MS Office 2007 formats:
     mime_map["docx"] = "application/vnd.openxmlformats-officedocument.wordprocessingml.document"; // Word 2007
     mime_map["dotx"] = "application/vnd.openxmlformats-officedocument.wordprocessingml.template"; // Word 2007 template
@@ -818,6 +871,25 @@ main(int argc, char **argv)
     mime_map["ppsx"] = "application/vnd.openxmlformats-officedocument.presentationml.slideshow"; // PowerPoint 2007 slideshow
     mime_map["potx"] = "application/vnd.openxmlformats-officedocument.presentationml.template"; // PowerPoint 2007 template
     mime_map["xps"] = "application/vnd.ms-xpsdocument";
+
+    // Macro-enabled variants - these appear to be the same formats as the
+    // above.  Currently we just treat them as the same mimetypes to avoid
+    // having to check for twice as many possible content-types.
+    // MS say: application/vnd.ms-word.document.macroEnabled.12
+    mime_map["docm"] = "application/vnd.openxmlformats-officedocument.wordprocessingml.document";
+    // MS say: application/vnd.ms-word.template.macroEnabled.12
+    mime_map["dotm"] = "application/vnd.openxmlformats-officedocument.wordprocessingml.template";
+    // MS say: application/vnd.ms-excel.sheet.macroEnabled.12
+    mime_map["xlsm"] = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";
+    // MS say: application/vnd.ms-excel.template.macroEnabled.12
+    mime_map["xltm"] = "application/vnd.openxmlformats-officedocument.spreadsheetml.template";
+    // MS say: application/vnd.ms-powerpoint.presentation.macroEnabled.12
+    mime_map["pptm"] = "application/vnd.openxmlformats-officedocument.presentationml.presentation";
+    // MS say: application/vnd.ms-powerpoint.slideshow.macroEnabled.12
+    mime_map["ppsm"] = "application/vnd.openxmlformats-officedocument.presentationml.slideshow";
+    // MS say: application/vnd.ms-powerpoint.presentation.macroEnabled.12
+    mime_map["potm"] = "application/vnd.openxmlformats-officedocument.presentationml.template";
+
     // Some other word processor formats:
     mime_map["doc"] = "application/msword";
     mime_map["dot"] = "application/msword"; // Word template
@@ -827,23 +899,28 @@ main(int argc, char **argv)
     mime_map["abw"] = "application/x-abiword"; // AbiWord
     mime_map["zabw"] = "application/x-abiword-compressed"; // AbiWord compressed
     mime_map["rtf"] = "text/rtf";
+
     // Other MS formats:
     mime_map["xls"] = "application/vnd.ms-excel";
     mime_map["xlb"] = "application/vnd.ms-excel";
     mime_map["xlt"] = "application/vnd.ms-excel"; // Excel template
     mime_map["ppt"] = "application/vnd.ms-powerpoint";
     mime_map["pps"] = "application/vnd.ms-powerpoint"; // Powerpoint slideshow
+
     // Perl:
     mime_map["pl"] = "text/x-perl";
     mime_map["pm"] = "text/x-perl";
     mime_map["pod"] = "text/x-perl";
+
     // TeX DVI:
     mime_map["dvi"] = "application/x-dvi";
+
     // DjVu:
     mime_map["djv"] = "image/vnd.djvu";
     mime_map["djvu"] = "image/vnd.djvu";
 
-    while ((getopt_ret = gnu_getopt_long(argc, argv, "hvd:D:U:M:l:s:pf", longopts, NULL)) != -1) {
+    while ((getopt_ret = gnu_getopt_long(argc, argv, "hvd:D:U:M:l:s:pfS",
+					 longopts, NULL)) != -1) {
 	switch (getopt_ret) {
 	case 'h': {
 	    cout << PROG_NAME" - "PROG_DESC"\n\n"
@@ -857,6 +934,7 @@ main(int argc, char **argv)
 "  -M, --mime-type          additional MIME mapping ext:type\n"
 "  -l, --depth-limit=LIMIT  set recursion limit (0 = unlimited)\n"
 "  -f, --follow             follow symbolic links\n"
+"  -S, --spelling           index data for spelling correction\n"
 "      --overwrite          create the database anew (the default is to update\n"
 "                           if the database already exists)" << endl;
 	    print_stemmer_help("     ");
@@ -924,6 +1002,9 @@ main(int argc, char **argv)
 		return 1;
 	    }
 	    break;
+	case 'S':
+	    spelling = true;
+	    break;
 	case ':': // missing param
 	    return 1;
 	case '?': // unknown option: FIXME -> char
@@ -972,10 +1053,22 @@ main(int argc, char **argv)
 		// + 1 so that db.get_lastdocid() is a valid subscript.
 		updated.resize(db.get_lastdocid() + 1);
 	    }
+	    try {
+		string ubound = db.get_value_upper_bound(VALUE_LASTMOD);
+		if (!ubound.empty()) 
+		    last_mod_max = binary_string_to_int(ubound);
+	    } catch (const Xapian::UnimplementedError &) {
+		numeric_limits<time_t> n;
+		last_mod_max = n.max();
+	    }
 	} else {
 	    db = Xapian::WritableDatabase(dbpath, Xapian::DB_CREATE_OR_OVERWRITE);
 	}
 
+	if (spelling) {
+	    indexer.set_database(db);
+	    indexer.set_flags(indexer.FLAG_SPELLING);
+	}
 	indexer.set_stemmer(stemmer);
 
 	index_directory(depth_limit, "/", mime_map);
@@ -990,7 +1083,7 @@ main(int argc, char **argv)
 		}
 	    }
 	}
-	db.flush();
+	db.commit();
 	// cout << "\n\nNow we have " << db.get_doccount() << " documents." << endl;
 	exitcode = 0;
     } catch (const Xapian::Error &e) {
