@@ -4,7 +4,7 @@
  * Copyright 2001 James Aylett
  * Copyright 2001,2002 Ananova Ltd
  * Copyright 2002 Intercede 1749 Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
  * Copyright 2008 Thomas Viehmann
  *
  * This program is free software; you can redistribute it and/or
@@ -31,14 +31,14 @@
 #include <set>
 #include <vector>
 
-#include <assert.h>
-#include <ctype.h>
+#include <cassert>
+#include <cctype>
 #include "safeerrno.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include "strcasecmp.h"
-#include <time.h>
+#include <ctime>
 
 #include "safeunistd.h"
 #include <sys/types.h>
@@ -53,10 +53,6 @@
 
 #include <cdb.h>
 
-#ifdef HAVE_PCRE
-#include <pcre.h>
-#endif
-
 #include "date.h"
 #include "datematchdecider.h"
 #include "utils.h"
@@ -64,8 +60,11 @@
 #include "query.h"
 #include "cgiparam.h"
 #include "loadfile.h"
+#include "str.h"
 #include "stringutils.h"
+#include "transform.h"
 #include "values.h"
+#include "weight.h"
 
 #include <xapian.h>
 
@@ -76,7 +75,7 @@ using Xapian::Utf8Iterator;
 using Xapian::Unicode::is_wordchar;
 
 #ifndef SNPRINTF
-#include <stdarg.h>
+#include <cstdarg>
 
 static int my_snprintf(char *str, size_t size, const char *format, ...)
 {
@@ -125,6 +124,8 @@ static string error_msg;
 
 static long int sec = 0, usec = -1;
 
+static bool suppress_http_headers = false;
+
 static const char DEFAULT_LOG_ENTRY[] =
 	"$or{$env{REMOTE_HOST},$env{REMOTE_ADDR},-}\t"
 	"[$date{$now,%d/%b/%Y:%H:%M:%S} +0000]\t"
@@ -236,7 +237,10 @@ set_probabilistic(const string &oldp)
     }
 
     try {
-	query = qp.parse_query(query_string);
+	unsigned f = qp.FLAG_DEFAULT;
+	if (option["spelling"] == "true")
+	    f |= qp.FLAG_SPELLING_CORRECTION;
+	query = qp.parse_query(query_string, f);
     } catch (Xapian::QueryParserError &e) {
 	error_msg = e.get_msg();
 	return BAD_QUERY;
@@ -312,6 +316,7 @@ void add_bterm(const string &term) {
 static void
 run_query()
 {
+    bool force_boolean = false;
     if (!filter_map.empty()) {
 	// OR together filters with the same prefix, then AND together
 	vector<Xapian::Query> filter_vec;
@@ -347,7 +352,7 @@ run_query()
 	    // to be THE query - filtering an empty query will give no
 	    // matches.
 	    std::swap(query, filter);
-	    if (enquire) enquire->set_weighting_scheme(Xapian::BoolWeight());
+	    if (enquire) force_boolean = true;
 	} else {
 	    query = Xapian::Query(Xapian::Query::OP_FILTER, query, filter);
 	}
@@ -377,6 +382,8 @@ run_query()
 
     if (!enquire || !error_msg.empty()) return;
 
+    set_weighting_scheme(*enquire, option, force_boolean);
+
     enquire->set_cutoff(threshold);
 
     if (sort_key != Xapian::BAD_VALUENO) {
@@ -799,14 +806,13 @@ CMD_split,
 CMD_stoplist,
 CMD_sub,
 CMD_substr,
+CMD_suggestion,
 CMD_terms,
 CMD_thispage,
 CMD_time,
 CMD_topdoc,
 CMD_topterms,
-#ifdef HAVE_PCRE
 CMD_transform,
-#endif
 CMD_uniq,
 CMD_unpack,
 CMD_unstem,
@@ -918,6 +924,7 @@ T(split,	   1, 2, N, 0), // split a string to give a list
 T(stoplist,	   0, 0, N, Q), // return list of stopped terms
 T(sub,		   2, 2, N, 0), // subtract
 T(substr,	   2, 3, N, 0), // substring
+T(suggestion,	   0, 0, N, Q), // misspelled word correction suggestion
 T(terms,	   0, 0, N, M), // list of matching terms
 T(thispage,	   0, 0, N, M), // page number of current page
 T(time,		   0, 0, N, M), // how long the match took (in seconds)
@@ -925,9 +932,7 @@ T(topdoc,	   0, 0, N, M), // first document on current page of hit list
 				// (counting from 0)
 T(topterms,	   0, 1, N, M), // list of up to N top relevance feedback terms
 				// (default 16)
-#ifdef HAVE_PCRE
 T(transform,	   3, 3, N, 0), // transform with a regexp
-#endif
 T(uniq,		   1, 1, N, 0), // removed duplicates from a sorted list
 T(unpack,	   1, 1, N, 0), // convert 4 byte big endian binary string to a number
 T(unstem,	   1, 1, N, Q), // return list of probabilistic terms from
@@ -1084,7 +1089,7 @@ eval(const string &fmt, const vector<string> &param)
 		vector<string>::const_iterator i;
 		for (i = args.begin(); i != args.end(); i++)
 		    total += string_to_int(*i);
-		value = int_to_string(total);
+		value = str(total);
 		break;
 	    }
 	    case CMD_addfilter:
@@ -1126,7 +1131,7 @@ eval(const string &fmt, const vector<string> &param)
 		break;
 	    }
 	    case CMD_collapsed: {
-		value = int_to_string(collapsed);
+		value = str(collapsed);
 		break;
 	    }
 	    case CMD_date:
@@ -1150,7 +1155,7 @@ eval(const string &fmt, const vector<string> &param)
 	    case CMD_dbsize: {
 		static Xapian::doccount dbsize;
 		if (!dbsize) dbsize = db.get_doccount();
-		value = int_to_string(dbsize);
+		value = str(dbsize);
 		break;
 	    }
 	    case CMD_def: {
@@ -1177,8 +1182,8 @@ eval(const string &fmt, const vector<string> &param)
 		if (denom == 0) {
 		    value = "divide by 0";
 		} else {
-		    value = int_to_string(string_to_int(args[0]) /
-					  string_to_int(args[1]));
+		    value = str(string_to_int(args[0]) /
+				string_to_int(args[1]));
 		}
 		break;
 	    }
@@ -1265,7 +1270,7 @@ eval(const string &fmt, const vector<string> &param)
 		    if (j == string::npos) j = l.size();
 		    if (j - i == s.length()) {
 			if (memcmp(s.data(), l.data() + i, j - i) == 0) {
-			    value = int_to_string(count);
+			    value = str(count);
 			    break;
 			}
 		    }
@@ -1279,11 +1284,11 @@ eval(const string &fmt, const vector<string> &param)
 		break;
 	    case CMD_freq:
 		try {
-		    value = int_to_string(mset.get_termfreq(args[0]));
+		    value = str(mset.get_termfreq(args[0]));
 		} catch (const Xapian::InvalidOperationError&) {
 		    // An MSet will raise this error if it's empty and not
 		    // associated with a search.
-		    value = int_to_string(db.get_termfreq(args[0]));
+		    value = str(db.get_termfreq(args[0]));
 		}
 		break;
             case CMD_ge:
@@ -1314,7 +1319,7 @@ eval(const string &fmt, const vector<string> &param)
 	    }
 	    case CMD_hit:
 		// 0-based mset index
-		value = int_to_string(hit_no);
+		value = str(hit_no);
 		break;
 	    case CMD_hitlist:
 #if 0
@@ -1351,7 +1356,7 @@ eval(const string &fmt, const vector<string> &param)
 		hit_no = 0;
 		break;
 	    case CMD_hitsperpage:
-		value = int_to_string(hits_per_page);
+		value = str(hits_per_page);
 		break;
 	    case CMD_hostname: {
 	        value = args[0];
@@ -1374,15 +1379,17 @@ eval(const string &fmt, const vector<string> &param)
 	        value = html_strip(args[0]);
 		break;
 	    case CMD_httpheader:
-		cout << args[0] << ": " << args[1] << endl;
-		if (!set_content_type && args[0].length() == 12 &&
-			strcasecmp(args[0].c_str(), "Content-Type") == 0) {
-		    set_content_type = true;
+		if (!suppress_http_headers) {
+		    cout << args[0] << ": " << args[1] << endl;
+		    if (!set_content_type && args[0].length() == 12 &&
+			    strcasecmp(args[0].c_str(), "Content-Type") == 0) {
+			set_content_type = true;
+		    }
 		}
 	        break;
 	    case CMD_id:
 		// document id
-		value = int_to_string(q0);
+		value = str(q0);
 		break;
 	    case CMD_if:
 		if (!args[0].empty())
@@ -1394,12 +1401,12 @@ eval(const string &fmt, const vector<string> &param)
 	        value = eval_file(args[0]);
 	        break;
 	    case CMD_last:
-		value = int_to_string(last);
+		value = str(last);
 		break;
 	    case CMD_lastpage: {
 		int l = mset.get_matches_estimated();
 		if (l > 0) l = (l - 1) / hits_per_page + 1;
-		value = int_to_string(l);
+		value = str(l);
 		break;
 	    }
             case CMD_le:
@@ -1411,7 +1418,7 @@ eval(const string &fmt, const vector<string> &param)
 		    value = "0";
 		} else {
 		    size_t length = count(args[0].begin(), args[0].end(), '\t');
-		    value = int_to_string(length + 1);
+		    value = str(length + 1);
 		}
 		break;
 	    case CMD_list: {
@@ -1520,7 +1527,7 @@ eval(const string &fmt, const vector<string> &param)
 		    int x = string_to_int(*i);
 		    if (x > val) val = x;
 	        }
-		value = int_to_string(val);
+		value = str(val);
 		break;
 	    }
 	    case CMD_min: {
@@ -1530,12 +1537,12 @@ eval(const string &fmt, const vector<string> &param)
 		    int x = string_to_int(*i);
 		    if (x < val) val = x;
 	        }
-		value = int_to_string(val);
+		value = str(val);
 		break;
 	    }
 	    case CMD_msize:
 		// number of matches
-		value = int_to_string(mset.get_matches_estimated());
+		value = str(mset.get_matches_estimated());
 		break;
 	    case CMD_msizeexact:
 		// is msize exact?
@@ -1548,8 +1555,8 @@ eval(const string &fmt, const vector<string> &param)
 		if (denom == 0) {
 		    value = "divide by 0";
 		} else {
-		    value = int_to_string(string_to_int(args[0]) %
-					  string_to_int(args[1]));
+		    value = str(string_to_int(args[0]) %
+				string_to_int(args[1]));
 		}
 		break;
 	    }
@@ -1558,7 +1565,7 @@ eval(const string &fmt, const vector<string> &param)
 		int total = string_to_int(*i++);
 		while (i != args.end())
 		    total *= string_to_int(*i++);
-		value = int_to_string(total);
+		value = str(total);
 		break;
 	    }
 	    case CMD_muldiv: {
@@ -1567,7 +1574,7 @@ eval(const string &fmt, const vector<string> &param)
 		    value = "divide by 0";
 		} else {
 		    int num = string_to_int(args[0]) * string_to_int(args[1]);
-		    value = int_to_string(num / denom);
+		    value = str(num / denom);
 		}
 		break;
 	    }
@@ -1615,7 +1622,7 @@ eval(const string &fmt, const vector<string> &param)
 		break;
 	    case CMD_percentage:
 		// percentage score
-		value = int_to_string(percent);
+		value = str(percent);
 		break;
 	    case CMD_prettyterm:
 		value = pretty_term(args[0]);
@@ -1633,7 +1640,7 @@ eval(const string &fmt, const vector<string> &param)
 		int start = string_to_int(args[0]);
 		int end = string_to_int(args[1]);
 	        while (start <= end) {
-		    value = value + int_to_string(start);
+		    value += str(start);
 		    if (start < end) value += '\t';
 		    start++;
 		}
@@ -1652,21 +1659,24 @@ eval(const string &fmt, const vector<string> &param)
 		map<Xapian::docid, bool>::iterator i = ticked.find(id);
 		if (i != ticked.end()) {
 		    i->second = false; // icky side-effect
-		    value = int_to_string(id);
+		    value = str(id);
 		}
 		break;
 	    }
 	    case CMD_relevants:	{
 		for (map <Xapian::docid, bool>::const_iterator i = ticked.begin();
 		     i != ticked.end(); i++) {
-		    if (i->second) value += int_to_string(i->first) + '\t';
+		    if (i->second) {
+			value += str(i->first);
+			value += '\t';
+		    }
 		}
 		if (!value.empty()) value.erase(value.size() - 1);
 		break;
 	    }
 	    case CMD_score:
 	        // Score (0 to 10)
-		value = int_to_string(percent / 10);
+		value = str(percent / 10);
 		break;
 	    case CMD_set:
 		option[args[0]] = args[1];
@@ -1753,8 +1763,7 @@ eval(const string &fmt, const vector<string> &param)
 		break;
 	    }
 	    case CMD_sub:
-		value = int_to_string(string_to_int(args[0]) -
-				      string_to_int(args[1]));
+		value = str(string_to_int(args[0]) - string_to_int(args[1]));
 		break;
 	    case CMD_substr: {
 		int start = string_to_int(args[1]);
@@ -1784,6 +1793,9 @@ eval(const string &fmt, const vector<string> &param)
 		value = args[0].substr(start, len);
 		break;
 	    }
+	    case CMD_suggestion:
+		value = qp.get_corrected_query_string();
+		break;
 	    case CMD_terms:
 		if (enquire) {
 		    // list of matching terms
@@ -1800,7 +1812,7 @@ eval(const string &fmt, const vector<string> &param)
 		}
 		break;
 	    case CMD_thispage:
-		value = int_to_string(topdoc / hits_per_page + 1);
+		value = str(topdoc / hits_per_page + 1);
 		break;
 	    case CMD_time:
 		if (usec != -1) {
@@ -1814,7 +1826,7 @@ eval(const string &fmt, const vector<string> &param)
 		break;
 	    case CMD_topdoc:
 		// first document on current page of hit list (counting from 0)
-		value = int_to_string(topdoc);
+		value = str(topdoc);
 		break;
 	    case CMD_topterms:
 		if (enquire) {
@@ -1858,42 +1870,9 @@ eval(const string &fmt, const vector<string> &param)
 		    if (!value.empty()) value.erase(value.size() - 1);
 		}
 		break;
-#ifdef HAVE_PCRE
-	    case CMD_transform: {
-		pcre *re;
-		const char *error;
-		int erroffset;
-		int offsets[30];
-		int matches;
-		re = pcre_compile(args[0].c_str(), 0, &error, &erroffset, NULL);
-		matches = pcre_exec(re, NULL, args[2].data(), args[2].size(),
-				    0, 0, offsets, 30);
-		if (matches > 0) {
-		    string::const_iterator i;
-		    value = args[2].substr(0, offsets[0]);
-		    for (i = args[1].begin(); i != args[1].end(); ++i) {
-			if (*i != '\\') {
-			    value += *i;
-			} else {
-			    ++i;
-			    if (i != args[1].end()) {
-				if (*i >= '0' && *i < '0' + matches) {
-				    int c = (*i - '0') * 2;
-				    value.append(args[2].substr(offsets[c],
-						offsets[c + 1] - offsets[c]));
-				} else {
-				    value += *i;
-				}
-			    }
-			}
-		    }
-		    value += args[2].substr(offsets[1]);
-		} else {
-		    value = args[2];
-		}
+	    case CMD_transform:
+		omegascript_transform(value, args);
 		break;
-	    }
-#endif
 	    case CMD_uniq: {
 		const string &list = args[0];
 		if (list.empty()) break;
@@ -1914,7 +1893,7 @@ eval(const string &fmt, const vector<string> &param)
 		break;
 	    }
 	    case CMD_unpack:
-		value = int_to_string(binary_string_to_int(args[0]));
+		value = str(binary_string_to_int(args[0]));
 		break;
 	    case CMD_unstem: {
 		const string &term = args[0];
@@ -2108,8 +2087,14 @@ void
 parse_omegascript()
 {
     try {
+	const char * p = getenv("SERVER_PROTOCOL");
+	if (p && strcmp(p, "INCLUDED") == 0) {
+	    // We're being included in another page, so suppress headers.
+	    suppress_http_headers = true;
+	}
+
 	std::string output = eval_file(fmtname);
-	if (!set_content_type) {
+	if (!set_content_type && !suppress_http_headers) {
 	    cout << "Content-Type: text/html" << std::endl;
 	}
 	cout << std::endl;
@@ -2117,7 +2102,7 @@ parse_omegascript()
     } catch (...) {
 	// Ensure the headers have been output so that any exception gets
 	// reported rather than giving a server error.
-	if (!set_content_type) {
+	if (!set_content_type && !suppress_http_headers) {
 	    cout << "Content-Type: text/html" << std::endl;
 	}
 	cout << std::endl;
