@@ -3,7 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 Sam Liddicott
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2009 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -36,8 +36,8 @@
 
 #include <cstdlib>
 #include "safeerrno.h"
-#include <stdio.h>
-#include <time.h>
+#include <cstdio>
+#include <ctime>
 #include "safeunistd.h"
 
 #include "commonhelp.h"
@@ -96,7 +96,7 @@ prefix_needs_colon(const string & prefix, unsigned ch)
 const char * action_names[] = {
     "bad", "new",
     "boolean", "date", "field", "hash", "index", "indexnopos", "load", "lower",
-    "truncate", "unhtml", "unique", "value", "valuenumeric", "weight"
+    "spell", "truncate", "unhtml", "unique", "value", "valuenumeric", "weight"
 };
 
 // For debugging:
@@ -107,7 +107,7 @@ public:
     typedef enum {
 	BAD, NEW,
 	BOOLEAN, DATE, FIELD, HASH, INDEX, INDEXNOPOS, LOAD, LOWER,
-	TRUNCATE, UNHTML, UNIQUE, VALUE, VALUENUMERIC, WEIGHT
+	SPELL, TRUNCATE, UNHTML, UNIQUE, VALUE, VALUENUMERIC, WEIGHT
     } type;
 private:
     type action;
@@ -235,6 +235,11 @@ parse_index_script(const string &filename)
 			    code = Action::LOAD;
 			}
 			break;
+		    case 's':
+			if (action == "spell") {
+			    code = Action::SPELL;
+			}
+			break;
 		    case 't':
 			if (action == "truncate") {
 			    code = Action::TRUNCATE;
@@ -300,16 +305,13 @@ parse_index_script(const string &filename)
 			if (val == "nopos") {
 			    // INDEX used to take an optional argument which
 			    // could be "nopos" to mean the same that
-			    // INDEXNOPOS now does.
-			    cout << filename << ':' << line_no
-				 << ": Warning: Index action '" << action
-				 << '=' << val << "' is deprecated - "
-				    "use action 'indexnopos' instead" << endl;
-			    // Translate this to allow older scripts to work
-			    // (this is safe to do since nopos isn't a sane
-			    // prefix value).
-			    code = Action::INDEXNOPOS;
-			    val = "";
+			    // INDEXNOPOS now does.  FIXME:1.3.0 remove this
+			    // error eventually
+			    cerr << filename << ':' << line_no
+				 << ": Support for 'index=nopos' has been "
+				    "removed - use 'indexnopos' instead"
+				 << endl;
+			    exit(1);
 			}
 			/* FALLTHRU */
 		    case Action::INDEXNOPOS:
@@ -380,6 +382,7 @@ parse_index_script(const string &filename)
 	    switch (action) {
 		case Action::HASH:
 		case Action::LOWER:
+		case Action::SPELL:
 		case Action::TRUNCATE:
 		case Action::UNHTML:
 		    done = false;
@@ -465,6 +468,9 @@ index_file(const char *fname, istream &stream,
 		value += line;
 	    }
 
+	    // Default to not indexing spellings.
+	    indexer.set_flags(Xapian::TermGenerator::flags(0));
+
 	    const vector<Action> &v = index_spec[field];
 	    string old_value = value;
 	    vector<Action>::const_iterator i;
@@ -540,6 +546,9 @@ index_file(const char *fname, istream &stream,
 		    case Action::TRUNCATE:
 			utf8_truncate(value, i->get_num_arg());
 			break;
+		    case Action::SPELL:
+			indexer.set_flags(indexer.FLAG_SPELLING);
+			break;
 		    case Action::UNHTML: {
 			MyHtmlParser p;
 			try {
@@ -598,7 +607,7 @@ again:
 			    // Hmm, what happened?
 			    cout << "Caught exception in UNIQUE!" << endl;
 			    cout << "E: " << e.get_msg() << endl;
-			    database.flush();
+			    database.commit();
 			    goto again;
 			}
 			break;
@@ -698,16 +707,17 @@ again:
 	}
     }
 
-    // Flush after each file to make sure all changes from that file make it in.
-    if (verbose) cout << "Flushing: " << endl;
-    database.flush();
+    // Commit after each file to make sure all changes from that file make it
+    // in.
+    if (verbose) cout << "Committing: " << endl;
+    database.commit();
 
     return true;
 }
 
 int
 main(int argc, char **argv)
-{
+try {
     // If the database already exists, default to updating not overwriting.
     int database_mode = Xapian::DB_CREATE_OR_OPEN;
     verbose = false;
@@ -773,55 +783,53 @@ main(int argc, char **argv)
 
     parse_index_script(argv[1]);
 
-    // Catch any Xapian::Error exceptions thrown.
-    try {
-	// Open the database.
-	Xapian::WritableDatabase database;
-	while (true) {
-	    try {
-		database = Xapian::WritableDatabase(argv[0], database_mode);
-		break;
-	    } catch (const Xapian::DatabaseLockError &) {
-		// Sleep and retry if we get a Xapian::DatabaseLockError -
-		// this just means that another process is updating the
-		// database.
-		cout << "Database locked ... retrying" << endl;
-		sleep(1);
-	    }
+    // Open the database.
+    Xapian::WritableDatabase database;
+    while (true) {
+	try {
+	    database = Xapian::WritableDatabase(argv[0], database_mode);
+	    break;
+	} catch (const Xapian::DatabaseLockError &) {
+	    // Sleep and retry if we get a Xapian::DatabaseLockError - this
+	    // just means that another process is updating the database.
+	    cout << "Database locked ... retrying" << endl;
+	    sleep(1);
 	}
-
-	Xapian::TermGenerator indexer;
-	indexer.set_stemmer(stemmer);
-
-	addcount = 0;
-	repcount = 0;
-	delcount = 0;
-
-	if (argc == 2) {
-	    // Read from stdin.
-	    index_file("<stdin>", cin, database, indexer);
-	} else {
-	    // Read file(s) listed on the command line.
-	    for (int i = 2; i < argc; ++i) {
-		ifstream stream(argv[i]);
-		if (stream) {
-		    index_file(argv[i], stream, database, indexer);
-		} else {
-		    cout << "Can't open file " << argv[i] << endl;
-		}
-	    }
-	}
-
-	cout << "records (added, replaced, deleted) = (" << addcount <<
-		", " << repcount << ", " << delcount << ")" << endl;
-    } catch (const Xapian::Error &error) {
-	cout << "Exception: " << error.get_msg() << endl;
-	exit(1);
-    } catch (const std::bad_alloc &) {
-	cout << "Exception: std::bad_alloc" << endl;
-	exit(1);
-    } catch (...) {
-	cout << "Unknown Exception" << endl;
-	exit(1);
     }
+
+    Xapian::TermGenerator indexer;
+    indexer.set_stemmer(stemmer);
+    // Set the database for spellings to be added to by the "spell" action.
+    indexer.set_database(database);
+
+    addcount = 0;
+    repcount = 0;
+    delcount = 0;
+
+    if (argc == 2) {
+	// Read from stdin.
+	index_file("<stdin>", cin, database, indexer);
+    } else {
+	// Read file(s) listed on the command line.
+	for (int i = 2; i < argc; ++i) {
+	    ifstream stream(argv[i]);
+	    if (stream) {
+		index_file(argv[i], stream, database, indexer);
+	    } else {
+		cout << "Can't open file " << argv[i] << endl;
+	    }
+	}
+    }
+
+    cout << "records (added, replaced, deleted) = (" << addcount << ", "
+	 << repcount << ", " << delcount << ")" << endl;
+} catch (const Xapian::Error &error) {
+    cout << "Exception: " << error.get_msg() << endl;
+    exit(1);
+} catch (const std::bad_alloc &) {
+    cout << "Exception: std::bad_alloc" << endl;
+    exit(1);
+} catch (...) {
+    cout << "Unknown Exception" << endl;
+    exit(1);
 }
