@@ -20,7 +20,7 @@
 
 #include <config.h>
 
-#include <stdlib.h>
+#include <cstdlib>
 #include <iostream>
 #include <string>
 
