@@ -23,8 +23,8 @@
 
 #include "utils.h"
 
-#include <stdio.h> // for sprintf/snprintf
-#include <stdlib.h>
+#include <cstdio> // for sprintf/snprintf
+#include <cstdlib>
 
 #include <string>
 
@@ -54,18 +54,6 @@ string_to_int(const string &s)
     return atoi(s.c_str());
 }
 
-string
-int_to_string(int val)
-{
-    CONVERT_TO_STRING("%d")
-}
-
-string
-long_to_string(long val)
-{
-    CONVERT_TO_STRING("%ld")
-}
-
 string
 double_to_string(double val)
 {
